/* rotatecubeGL.cpp
 * Application affichant un cube tournant sur lui-mme dans
 * une fentre Windows avec OpenGL	
 * Touches:
 * 	1 - Activation/Arrt de l'antialiasing
 *	2 - Activation/Arrt du mode filaire
 *	F1 - Mode plein cran ou fentr
 *	ESC - Quitter
 *
 * Auteur: Vouriot Thierry
 * Web: http://thierry.vouriot.free.fr
 * Email: thierry.vouriot@free.fr
 * Date: 11 septembre 2002
 */

#include "initGL.h"

#pragma comment( lib, "opengl32.lib" )			// recherche de OpenGL32.lib pendant l'dition de liens
#pragma comment( lib, "glu32.lib" )			// recherche de Glu32.lib pendant l'dition de liens

#define TITLE "RotateCubeGL (Vouriot Thierry)"		// titre du programme
 
GL_Window*	g_window;	// fentre OpenGL
Keys*		g_keys;		// touches clavier

/* Variables pour la rotation du cube */
float rotX = 0.0f, rotZ = 0.0, rotY = 0.0f;

// prototypes
GLvoid DrawCube(float x, float y, float z, float radius);

/* Initialisation de l'utilisateur et d'OpenGL */
bool Initialize(GL_Window* window, Keys* keys)
{
	g_window = window;
	g_keys = keys;
	SetWindowText(g_window->hWnd, TITLE);

	// OpenGL initialisation
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_FLAT);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST); // amliore les calculs de perspective

	return true;												// Return TRUE (Initialization Successful)
}

/* Appele juste avant la destruction de la fentre,
 * Libration mmoire, ...etc... 
 */
void Deinitialize(void)
{
	// rien  faire
}

/* Appele avant chaque affichage d'image,
 * Teste des touches clavier,
 * Calcule du frame rate  (milliseconds est le temps entre 2 affichages d'images),
 * Maj des diffrentes variables, ...etc...
 */
void Update(DWORD milliseconds)
{
	static bool antialias = false;    // antialiasing
  	static GLenum polyMode = GL_FILL;  // polygone mode
	
	// teste des touches du clavier
	if (g_keys->keyDown[VK_ESCAPE] == true)	// echap on quitte l'application
	{
		TerminateApplication(g_window);
	}
	if (g_keys->keyDown[VK_F1] == true)	// F1 basculement fullscreen/windowed
	{
		g_keys->keyDown[VK_F1] = false;
		ToggleFullscreen(g_window);
	}
	if (g_keys->keyDown['1'] == true)	// 1 antialiasing
	{
		g_keys->keyDown['1'] = false;
		antialias = !antialias;
          	// activation de l'antialiasing
          	if (antialias)
          	{
            		glEnable(GL_LINE_SMOOTH);
            		glEnable(GL_POLYGON_SMOOTH);
            		glEnable(GL_BLEND);
          	}
          	// arrt de l'antialiasing
          	else
          	{
            		glDisable(GL_LINE_SMOOTH);
            		glDisable(GL_POLYGON_SMOOTH);
            		glDisable(GL_BLEND);
          	}
	}
	if (g_keys->keyDown['2'] == true)	// 2 mode filaire ou plein
	{
		g_keys->keyDown['2'] = false;
		// passage du mode filaire au mode plein ou inversement
		polyMode == GL_FILL ? polyMode = GL_LINE : polyMode = GL_FILL;
		glPolygonMode(GL_FRONT_AND_BACK, polyMode);
	}

	// permet de faire tourner le cube toujours  la mme vitesse
	// mme si le nombre d'images par secondes est plus faible ou plus lev
	rotX += (float)(milliseconds) / 12.0f;
	rotY += (float)(milliseconds) / 12.0f;
	rotZ += (float)(milliseconds) / 10.0f;
	
	if (rotX >= 360.0) rotX = 0.0;
	if (rotY >= 360.0) rotY = 0.0;
	if (rotZ >= 360.0) rotZ = 0.0;	
}

/* Affichage de la scne */
void DisplayScene(void) 
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	// 	  Position      View	   Up Vector
	gluLookAt(0, 0, 6,     0, 0, 0,     0, 1, 0);

	// Rotation
	glRotatef(rotX, 1.0f, 0.0f, 0.0f);
	glRotatef(rotY, 0.0f, 1.0f, 0.0f);
	glRotatef(rotZ, 0.0f, 0.0f, 1.0f);

	DrawCube(-1, -1, -1, 2);
		
	glFlush();
}

/* Dessine un cube multicolor de taille radius  la postion x, y, z */
GLvoid DrawCube(float x, float y, float z, float radius)
{
	glPushMatrix();
		glTranslatef(x, y, z);
		glBegin(GL_QUADS);
		
			// face arrire
			glColor3ub(0, 255, 255);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, radius, 0.0f);
			glVertex3f(radius, radius, 0.0f); 
			glVertex3f(radius, 0.0f, 0.0f);

			// face avant
			glColor3ub(0, 255, 0);   
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(0.0f, radius, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, 0.0f, radius);

			// face de dessous
			glColor3ub(255, 0, 0);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(radius, 0.0f, radius); 
			glVertex3f(radius, 0.0f, 0.0f);
			
			// face de dessus
			glColor3ub(0, 0, 255);   
			glVertex3f(0.0f, radius, 0.0f);
			glVertex3f(0.0f, radius, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, radius, 0.0f);

			// face gauche
			glColor3ub(255, 0, 255);   
			glVertex3f(0.0f, 0.0f, 0.0f);
			glVertex3f(0.0f, 0.0f, radius);
			glVertex3f(0.0f, radius, radius); 
			glVertex3f(0.0f, radius, 0.0f);

			// face droite
			glColor3ub(255, 255, 0);   
			glVertex3f(radius, 0.0f, 0.0f);
			glVertex3f(radius, 0.0f, radius);
			glVertex3f(radius, radius, radius); 
			glVertex3f(radius, radius, 0.0f);
			glEnd();
	glPopMatrix();
}
